package sun.security.krb5.internal.tools;

import sun.security.krb5.internal.Krb5;
import sun.security.krb5.RealmException;
import sun.security.krb5.internal.ccache.Credentials;
import java.io.IOException;
import java.util.Arrays;
import sun.security.krb5.internal.ccache.CredentialsCache;
import sun.security.krb5.internal.HostAddresses;
import sun.security.krb5.PrincipalName;
import sun.security.krb5.Config;
import sun.security.krb5.internal.KDCOptions;
import sun.security.util.Password;
import sun.security.krb5.KrbAsReqBuilder;
import java.io.File;
import javax.security.auth.kerberos.KeyTab;
import sun.security.krb5.KrbException;

public class Kinit
{
    private KinitOptions options;
    private static final boolean DEBUG;
    
    public static void main(String[] array) {
        try {
            Kinit kinit = new Kinit(array);
        }
        catch (Exception obj) {
            String str;
            if ((obj instanceof KrbException)) {
                str = ((KrbException)obj).krbErrorMessage() + " " + ((KrbException)obj).returnCodeMessage();
            }
            else {
                str = obj.getMessage();
            }
            if (str != null) {
                System.err.println("Exception: " + str);
            }
            else {
                System.out.println("Exception: " + obj);
            }
            obj.printStackTrace();
            System.exit(-1);
        }
    }
    
    private Kinit(String[] array) throws IOException, RealmException, KrbException {
        if (array == null || array.length == 0) {
            this.options = new KinitOptions();
        }
        else {
            this.options = new KinitOptions(array);
        }
        String string = null;
        PrincipalName principal = this.options.getPrincipal();
        if (principal != null) {
            string = principal.toString();
        }
        if (Kinit.DEBUG) {
            System.out.println("Principal is " + principal);
        }
        char[] a = this.options.password;
        Object o;
        if (!this.options.useKeytabFile()) {
            if (string == null) {
                throw new IllegalArgumentException(" Can not obtain principal name");
            }
            if (a == null) {
                System.out.print("Password for " + string + ":");
                System.out.flush();
                a = Password.readPassword(System.in);
                if (Kinit.DEBUG) {
                    System.out.println(">>> Kinit console input " + new String(a));
                }
            }
            o = new KrbAsReqBuilder(principal, a);
        }
        else {
            if (Kinit.DEBUG) {
                System.out.println(">>> Kinit using keytab");
            }
            if (string == null) {
                throw new IllegalArgumentException("Principal name must be specified.");
            }
            String keytabFileName = this.options.keytabFileName();
            if (keytabFileName != null && Kinit.DEBUG) {
                System.out.println(">>> Kinit keytab file name: " + keytabFileName);
            }
            o = new KrbAsReqBuilder(principal, (keytabFileName == null) ? KeyTab.getInstance() : KeyTab.getInstance(new File(keytabFileName)));
        }
        KDCOptions options = new KDCOptions();
        setOptions(1, (int)this.options.forwardable, options);
        setOptions(3, (int)this.options.proxiable, options);
        o.setOptions(options);
        String s = this.options.getKDCRealm();
        if (s == null) {
            s = Config.getInstance().getDefaultRealm();
        }
        if (Kinit.DEBUG) {
            System.out.println(">>> Kinit realm name is " + s);
        }
        o.setTarget(PrincipalName.tgsService(s, s));
        if (Kinit.DEBUG) {
            System.out.println(">>> Creating KrbAsReq");
        }
        if (this.options.getAddressOption()) {
            o.setAddresses(HostAddresses.getLocalAddresses());
        }
        o.action();
        Credentials cCreds = o.getCCreds();
        o.destroy();
        CredentialsCache create = CredentialsCache.create(principal, this.options.cachename);
        if (create == null) {
            throw new IOException("Unable to create the cache file " + this.options.cachename);
        }
        create.update(cCreds);
        create.save();
        if (this.options.password == null) {
            System.out.println("New ticket is stored in cache file " + this.options.cachename);
        }
        else {
            Arrays.fill(this.options.password, '0');
        }
        if (a != null) {
            Arrays.fill(a, '0');
        }
        this.options = null;
        return;
    }
    
    private static void setOptions(int n, int n2, KDCOptions kdcOptions) {
        switch (n2) {
            case 0: {
                break;
            }
            case -1: {
                kdcOptions.set(n, false);
                break;
            }
            case 1: {
                kdcOptions.set(n, true);
                break;
            }
        }
    }
    
    static {
        DEBUG = Krb5.DEBUG;
    }
}
